@@ -1,5 +1,5 @@
 import logging
-import os
+from pathlib import Path
 from typing import (
     Any,
     Awaitable,
@@ -14,12 +14,19 @@ from typing import (
     Union,
 )
 
+from guardpost.asynchronous.authentication import AuthenticationStrategy
+from guardpost.asynchronous.authorization import AuthorizationStrategy
+from guardpost.authorization import Policy, UnauthorizedError
+from guardpost.common import AuthenticatedRequirement
+from rodi import Container, Services
+
 from blacksheep.baseapp import BaseApplication
 from blacksheep.common.files.asyncfs import FilesHandler
 from blacksheep.contents import ASGIContent
 from blacksheep.messages import Request, Response
 from blacksheep.middlewares import get_middlewares_chain
 from blacksheep.scribe import send_asgi_response
+from blacksheep.server.asgi import get_request_url_from_scope
 from blacksheep.server.authentication import (
     AuthenticateChallenge,
     get_authentication_middleware,
@@ -33,23 +40,15 @@ from blacksheep.server.authorization import (
 from blacksheep.server.bindings import ControllerParameter
 from blacksheep.server.controllers import router as controllers_router
 from blacksheep.server.cors import CORSPolicy, CORSStrategy, get_cors_middleware
+from blacksheep.server.env import EnvironmentSettings
 from blacksheep.server.errors import ServerErrorDetailsHandler
 from blacksheep.server.files import ServeFilesOptions
 from blacksheep.server.files.dynamic import serve_files_dynamic
 from blacksheep.server.normalization import normalize_handler, normalize_middleware
-from blacksheep.server.routing import RegisteredRoute, Router, RoutesRegistry
-from blacksheep.sessions import (
-    Encryptor,
-    SessionSerializer,
-    Signer,
-    SessionMiddleware,
-)
+from blacksheep.server.responses import _ensure_bytes
+from blacksheep.server.routing import Mount, RegisteredRoute, Router, RoutesRegistry
+from blacksheep.sessions import Encryptor, SessionMiddleware, SessionSerializer, Signer
 from blacksheep.utils import ensure_bytes, join_fragments
-from guardpost.asynchronous.authentication import AuthenticationStrategy
-from guardpost.asynchronous.authorization import AuthorizationStrategy
-from guardpost.authorization import Policy, UnauthorizedError
-from guardpost.common import AuthenticatedRequirement
-from rodi import Container, Services
 
 __all__ = ("Application",)
 
@@ -72,25 +71,51 @@ def get_default_headers_middleware(
 
 class ApplicationEvent:
     def __init__(self, context: Any) -> None:
-        self.__handlers: List[Callable[..., Any]] = []
+        self._handlers: List[Callable[..., Any]] = []
         self.context = context
 
     def __iadd__(self, handler: Callable[..., Any]) -> "ApplicationEvent":
-        self.__handlers.append(handler)
+        self._handlers.append(handler)
         return self
 
     def __isub__(self, handler: Callable[..., Any]) -> "ApplicationEvent":
-        self.__handlers.remove(handler)
+        self._handlers.remove(handler)
         return self
 
     def __len__(self) -> int:
-        return len(self.__handlers)
+        return len(self._handlers)
+
+    def __call__(self, *args) -> Any:
+        if args:
+            self.__iadd__(args[0])
+            return args[0]
+
+        def decorator(fn):
+            self.__iadd__(fn)
+            return fn
+
+        return decorator
 
     async def fire(self, *args: Any, **keywargs: Any) -> None:
-        for handler in self.__handlers:
+        for handler in self._handlers:
             await handler(self.context, *args, **keywargs)
 
 
+class ApplicationSyncEvent(ApplicationEvent):
+    """
+    ApplicationEvent whose subscribers must be synchronous functions.
+    """
+
+    def fire_sync(self, *args: Any, **keywargs: Any) -> None:
+        for handler in self._handlers:
+            handler(self.context, *args, **keywargs)
+
+    async def fire(self, *args: Any, **keywargs: Any) -> None:
+        raise TypeError(
+            "The event handlers in this ApplicationEvent must be synchronous!"
+        )
+
+
 class ApplicationStartupError(RuntimeError):
     ...
 
@@ -112,6 +137,16 @@ class ApplicationAlreadyStartedCORSError(TypeError):
         )
 
 
+def _extend(obj, cls):
+    """Applies a mixin to an instance of a class."""
+    base_cls = obj.__class__
+    base_cls_name = obj.__class__.__name__
+    obj.__class__ = type(base_cls_name, (cls, base_cls), {})
+
+
+env_settings = EnvironmentSettings()
+
+
 class Application(BaseApplication):
     def __init__(
         self,
@@ -120,13 +155,16 @@ class Application(BaseApplication):
         services: Optional[Container] = None,
         debug: bool = False,
         show_error_details: Optional[bool] = None,
+        mount: Optional[Mount] = None,
     ):
         if router is None:
             router = Router()
         if services is None:
             services = Container()
         if show_error_details is None:
-            show_error_details = bool(os.environ.get("APP_SHOW_ERROR_DETAILS", False))
+            show_error_details = env_settings.show_error_details
+        if mount is None:
+            mount = Mount()
         super().__init__(show_error_details, router)
 
         self.services: Container = services
@@ -141,11 +179,21 @@ class Application(BaseApplication):
         self.on_start = ApplicationEvent(self)
         self.after_start = ApplicationEvent(self)
         self.on_stop = ApplicationEvent(self)
+        self.on_middlewares_configuration = ApplicationSyncEvent(self)
         self.started = False
         self.controllers_router: RoutesRegistry = controllers_router
         self.files_handler = FilesHandler()
         self.server_error_details_handler = ServerErrorDetailsHandler()
         self._session_middleware: Optional[SessionMiddleware] = None
+        self._mount = mount
+
+    def mount(self, path: str, app: Callable) -> None:
+        self._mount.mount(path, app)
+
+        if len(self._mount.mounted_apps) == 1:
+            # the first time a mount is configured, extend the application
+            # to use mounts when handling web requests
+            self.extend(MountMixin)
 
     @property
     def service_provider(self) -> Services:
@@ -226,7 +274,7 @@ class Application(BaseApplication):
         # stop the chain of middlewares and prevent extra logic from executing for
         # preflight requests (e.g. authentication logic)
         @self.router.options("*")
-        async def options_handler(request):
+        async def options_handler(request: Request) -> Response:
             return Response(404)
 
         # User defined catch-all OPTIONS request handlers are not supported when the
@@ -262,6 +310,7 @@ class Application(BaseApplication):
         if not self._cors_strategy:
             self.use_cors()
 
+        assert self._cors_strategy is not None
         self._cors_strategy.add_policy(
             policy_name,
             CORSPolicy(
@@ -282,6 +331,10 @@ class Application(BaseApplication):
                 "The application is already running, configure authentication "
                 "before starting the application"
             )
+
+        if self._authentication_strategy:
+            return self._authentication_strategy
+
         if not strategy:
             strategy = AuthenticationStrategy()
 
@@ -297,6 +350,9 @@ class Application(BaseApplication):
                 "before starting the application"
             )
 
+        if self._authorization_strategy:
+            return self._authorization_strategy
+
         if not strategy:
             strategy = AuthorizationStrategy()
 
@@ -328,11 +384,12 @@ class Application(BaseApplication):
         return decorator
 
     def exception_handler(
-            self, exception: Union[int, Type[Exception]]
+        self, exception: Union[int, Type[Exception]]
     ) -> Callable[..., Any]:
         """
         Registers an exception handler function in the application exception handler.
         """
+
         def decorator(f):
             self.exceptions_handlers[exception] = f
             return f
@@ -341,7 +398,7 @@ class Application(BaseApplication):
 
     def serve_files(
         self,
-        source_folder: str,
+        source_folder: Union[str, Path],
         *,
         discovery: bool = False,
         cache_time: int = 10800,
@@ -540,6 +597,8 @@ class Application(BaseApplication):
                 0, get_default_headers_middleware(self._default_headers)
             )
 
+        self.on_middlewares_configuration.fire_sync()
+
         self._normalize_middlewares()
 
         if self.middlewares:
@@ -548,6 +607,12 @@ class Application(BaseApplication):
     def build_services(self):
         self._service_provider = self.services.build_provider()
 
+    def extend(self, mixin) -> None:
+        """
+        Extends the class with additional features, applying the given mixin class.
+        """
+        _extend(self, mixin)
+
     async def start(self):
         if self.started:
             return
@@ -593,8 +658,12 @@ class Application(BaseApplication):
         assert scope["type"] == "http"
 
         request = Request.incoming(
-            scope["method"], scope["raw_path"], scope["query_string"], scope["headers"]
+            scope["method"],
+            scope["raw_path"],
+            scope["query_string"],
+            list(scope["headers"]),
         )
+
         request.scope = scope
         request.content = ASGIContent(receive)
 
@@ -603,3 +672,50 @@ class Application(BaseApplication):
 
         request.scope = None  # type: ignore
         request.content.dispose()
+
+
+class MountMixin:
+    _mount: Mount
+
+    def handle_mount_path(self, scope, route_match):
+        assert route_match.values is not None
+        tail = route_match.values.get("tail")
+        assert tail is not None
+        tail = "/" + tail
+
+        scope["path"] = tail
+        scope["raw_path"] = tail.encode("utf8")
+
+    async def _handle_redirect_to_mount_root(self, scope, send):
+        """
+        A request to the path "https://.../{mount_path}" must result in a
+        307 Temporary Redirect to the root of the mount: "https://.../{mount_path}/"
+        including a trailing slash.
+        """
+        response = Response(
+            307,
+            [
+                (
+                    b"Location",
+                    _ensure_bytes(
+                        f"{get_request_url_from_scope(scope, trailing_slash=True)}"
+                    ),
+                )
+            ],
+        )
+        await send_asgi_response(response, send)
+
+    async def __call__(self, scope, receive, send):
+        if scope["type"] == "lifespan":
+            return await super()._handle_lifespan(receive, send)  # type: ignore
+
+        for route in self._mount.mounted_apps:  # type: ignore
+            route_match = route.match(scope["raw_path"])
+            if route_match:
+                raw_path = scope["raw_path"]
+                if raw_path == route.pattern.rstrip(b"/*") and scope["type"] == "http":
+                    return await self._handle_redirect_to_mount_root(scope, send)
+                self.handle_mount_path(scope, route_match)
+                return await route.handler(scope, receive, send)
+
+        return await super().__call__(scope, receive, send)  # type: ignore
