@@ -1,5 +1,3 @@
-from blacksheep.utils import join_fragments
-from blacksheep.server.authorization import allow_anonymous
 import html
 import os
 from pathlib import Path
@@ -10,8 +8,10 @@ from blacksheep import HTMLContent, Request, Response
 from blacksheep.common.files.asyncfs import FilesHandler
 from blacksheep.common.files.pathsutils import get_file_extension_from_name
 from blacksheep.exceptions import NotFound
+from blacksheep.server.authorization import allow_anonymous
 from blacksheep.server.resources import get_resource_file_content
 from blacksheep.server.routing import Route, Router
+from blacksheep.utils import join_fragments
 
 from . import get_default_extensions, get_response_for_file, validate_source_path
 
@@ -78,7 +78,7 @@ def get_files_list_html_response(
         full_rel_path = html.escape(
             join_fragments(root_path, parent_folder_path, rel_path)
         )
-        info_lines.append(f'<li><a href="/{full_rel_path}">{rel_path}</a></li>')
+        info_lines.append(f'<li><a href="{full_rel_path}">{rel_path}</a></li>')
     info = "".join(info_lines)
     p = []
     whole_p = [root_path]
@@ -86,7 +86,7 @@ def get_files_list_html_response(
         if fragment:
             whole_p.append(html.escape(fragment))
             fragment_path = "/".join(whole_p)
-            p.append(f'<a href="/{fragment_path}">{html.escape(fragment)}</a>')
+            p.append(f'<a href="{fragment_path}">{html.escape(fragment)}</a>')
 
     # TODO: use chunked encoding here, yielding HTML fragments
     return Response(
