@@ -25,13 +25,14 @@ from typing import (
     Union,
 )
 
+from essentials.json import dumps
+from openapidocs.common import Format, OpenAPIRoot, Serializer
+
 from blacksheep.messages import Request
-from blacksheep.server.application import Application
+from blacksheep.server.application import Application, ApplicationSyncEvent
 from blacksheep.server.authorization import allow_anonymous
 from blacksheep.server.files.static import get_response_for_static_content
-from blacksheep.server.responses import FriendlyEncoder
 from blacksheep.server.routing import Route, Router
-from openapidocs.common import Format, OpenAPIRoot, Serializer
 
 from .ui import SwaggerUIProvider, UIOptions, UIProvider
 
@@ -131,6 +132,21 @@ class OpenAPIEndpointException(Exception):
     pass
 
 
+class OpenAPIEvent(ApplicationSyncEvent):
+    pass
+
+
+class OpenAPIEvents:
+    on_operation_created: OpenAPIEvent
+    on_docs_created: OpenAPIEvent
+    on_paths_created: OpenAPIEvent
+
+    def __init__(self, context) -> None:
+        self.on_operation_created = OpenAPIEvent(context)
+        self.on_docs_created = OpenAPIEvent(context)
+        self.on_paths_created = OpenAPIEvent(context)
+
+
 class APIDocsHandler(Generic[OpenAPIRootType], ABC):
     """
     Provides methods to handle the documentation for an API.
@@ -155,6 +171,9 @@ class APIDocsHandler(Generic[OpenAPIRootType], ABC):
         self.preferred_format = preferred_format
         self.anonymous_access = anonymous_access
         self.ui_providers: List[UIProvider] = [SwaggerUIProvider(ui_path)]
+        self._types_schemas = {}
+        self.events = OpenAPIEvents(self)
+        self.handle_optional_response_with_404 = True
 
     def __call__(
         self,
@@ -190,6 +209,27 @@ class APIDocsHandler(Generic[OpenAPIRootType], ABC):
 
         return decorator
 
+    def register(self, schema) -> Any:
+        """
+        Registers a schema for a class. When the documentation handler needs
+        to obtain a schema for the decorated type, it uses the explicity schema rather
+        than an auto-generated schema.
+        """
+
+        def class_decorator(cls):
+            self.set_type_schema(cls, schema)
+            return cls
+
+        return class_decorator
+
+    def set_type_schema(self, object_type, schema) -> None:
+        """
+        Sets a schema to be used for a class. When the documentation handler needs
+        to obtain a schema for the decorated type, it uses the explicity schema rather
+        than an auto-generated schema.
+        """
+        self._types_schemas[object_type] = schema
+
     def get_handler_docs(self, obj: Any) -> Optional[EndpointDocs]:
         return self._handlers_docs.get(obj)
 
@@ -330,7 +370,7 @@ class APIDocsHandler(Generic[OpenAPIRootType], ABC):
         This method is used to ensure that YAML representations of objects look
         exactly the same as JSON representations.
         """
-        return json.loads(json.dumps(value, cls=FriendlyEncoder))
+        return json.loads(dumps(value))
 
     @abstractmethod
     def generate_documentation(self, app: Application) -> OpenAPIRootType:
